# !/usr/bin/python3
# -*-coding:utf-8-*-
# Author: zhou jun wei
# CreatDate: 2021/12/3 15:37
import io
from PIL import Image

def is_user_data_type(data):
    if isinstance(data, dict):
        if isinstance(data["图片"], str):
            if "<"  in data.get('图片') or '>' in data.get("图片"):
                raise TypeError('传入的路径不可以携带<,> or 传入图片,不是html')
        if data.get("网站") and data.get("图片"):
            return data
        else:
            raise TypeError('兄弟,你皮呐？ 在皮揍你，给我传空值？')
    else:
        raise TypeError('data 输入值必须是 dict (字典) ,请输入 {"img":"","domain",""}')



def credit_img_util(data):
    im = Image.open(io.BytesIO(data["图片"]))
    im.seek(8)
    img_byte = io.BytesIO()
    im.save(img_byte, format='PNG')
    binary_content = img_byte.getvalue()
    return {"网站": data['网站'], "图片": binary_content}

def is_user_model(data):
    if data["网站"] == "中国海关企业进出口信用信息公示平台":   # http://credit.customs.gov.cn/
        if isinstance(data["图片"], str):
            data = credit_img_util(data)
        elif isinstance(data["图片"], bytes):
            data = credit_img_util(data)
        else:
            raise TypeError('图必须是路径或字节')
        return data
    else:
        raise TypeError('现在只有 中国海关企业进出口信用信息公示平台 的验证码，静待作者后续版本升级')



import torchvision.transforms as T
import torch
from zjwocr.model import LightCNN
from zjwocr.img_utils import to_rgb, loadimg
from zjwocr.model_utils import credit_customs_gov_cn_label_map, credit_customs_gov_cn_num
from zjwocr.user_utils import is_user_data_type, is_user_model


DEVICE = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
model = LightCNN(output=len(credit_customs_gov_cn_label_map) * credit_customs_gov_cn_num)
if torch.cuda.is_available():
    model = model.to(DEVICE)
    model.load_state_dict(torch.load("./models/credit_customs_gov_cn.model"))
    model.eval()
else:
    model = model.to(DEVICE)
    model.load_state_dict(torch.load("./models/credit_customs_gov_cn.model", map_location=DEVICE))
    model.eval()


def ocr_api(data):
    """
    :param data: data = {'图片': '','网站': '中国海关企业进出口信用信息公示平台'}
    :return: 返回值：识别出的数字或者英文字母
    """
    data = is_user_data_type(data)
    data = is_user_model(data)
    binary = data["图片"]
    try:
        im = loadimg(binary, None)
        im = T.Compose([
            to_rgb,
            T.ToPILImage(),
            T.Resize((128, 128)),
            T.ToTensor(),
        ])(im)
        im = im.to(DEVICE)
        im = im.unsqueeze(0)
        result = model(im).view(credit_customs_gov_cn_num, len(credit_customs_gov_cn_label_map))
        predict = ''.join([credit_customs_gov_cn_label_map[i] for i in list(torch.argmax(result.cpu(), dim=1))])
    except:
        predict = ""
        raise TypeError('这是一个意外，报错。')
        
    return {"code": predict}








# !/usr/bin/python3
# -*-coding:utf-8-*-
# Author: zhou jun wei
# CreatDate: 2021/12/3 15:38

import cv2, numpy

def to_rgb(im):
    """
    图片转 RGB
    :param im: 图片 np.array
    :return: 图片 np.array
    """
    if len(im.shape) == 2:  # 灰度
        return cv2.cvtColor(im, cv2.COLOR_GRAY2RGB)
    elif im.shape[2] == 4:  # PNG (RGBA)
        return cv2.cvtColor(im, cv2.COLOR_RGBA2RGB)
    assert len(im.shape) == 3 and im.shape[2] == 3  # 如果 Channel 不为3, 肯定出问题了
    return im


def loadimg(path_or_bytes, size=None):
    """
    加载图片
    :param path_or_bytes: 图片目录 / 字节
    :param size: 缩放图片 (width, height)
    :return: 图片 numpy.array (height, width, dim)
    """
    if isinstance(path_or_bytes, str):
        im = numpy.fromfile(path_or_bytes, dtype=numpy.uint8)
    elif isinstance(path_or_bytes, bytes):
        im = numpy.frombuffer(path_or_bytes, dtype=numpy.uint8)
    else:
        raise TypeError("图必须是路径或字节")

    im = cv2.imdecode(im, -1)
    if size is not None:
        im = cv2.resize(im, size, interpolation=cv2.INTER_AREA)
    return im


# !/usr/bin/python3
# -*-coding:utf-8-*-
# Author: zhou jun wei
# CreatDate: 2021/12/3 14:39
import torch

class CNNBlock(torch.nn.Module):
    def __init__(self, in_features, out_features, kernel, strides):
        super(CNNBlock, self).__init__()
        self.conv = torch.nn.Conv2d(in_features, out_features, kernel, strides[0])
        self.bn = torch.nn.BatchNorm2d(out_features, momentum=0.9)
        self.relu = torch.nn.LeakyReLU(0.01)
        self.pool = torch.nn.MaxPool2d(2, strides[1])

        torch.nn.init.xavier_uniform_(self.conv.weight, gain=1)

    def forward(self, x):
        x = self.conv(x)
        x = self.bn(x)
        x = self.relu(x)
        x = self.pool(x)

        return x


class LightCNN(torch.nn.Module):

    def __init__(self, image_shape=(128, 128), output=100):
        super(LightCNN, self).__init__()
        self.image_shape = image_shape

        self.cnn = torch.nn.Sequential(
            CNNBlock(3, 32, 7, (1, 1)),
            CNNBlock(32, 64, 5, (1, 2)),
            CNNBlock(64, 128, 3, (1, 2)),
            CNNBlock(128, 128, 3, (1, 2)),
            CNNBlock(128, 64, 3, (1, 2))
        )

        self.linear = torch.nn.Sequential(
            torch.nn.Linear(self._cal_shape(), output)
        )

    def _cal_shape(self):
        x = torch.zeros((1, 3) + self.image_shape)
        return self.cnn(x).flatten().shape[0]

    def forward(self, x):
        batch_size = x.shape[0]

        x = self.cnn(x)
        x = x.view(batch_size, -1)
        x = self.linear(x)
        return x



# !/usr/bin/python3
# -*-coding:utf-8-*-
# Author: zhou jun wei
# CreatDate: 2021/12/3 11:38

from setuptools import find_packages, setup
# version = '0.0.1'
# author = 'zhoujunwei'


setup(
    name='zjwocr',  #　包的名字
    version='0.0.3',
    description='test',
    author='周军威',
    author_email='505104341@qq.com',
    url='https://github.com/zjw505104341/ocrimg',
    # packages=find_packages(where='.', exclude=(), include=('*',)),
    packages=["zjwocr"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'torch',
        'torchaudio',
        'torchvision'
        'numpy'
        'opencv-python'
        'Pillow'
        'requests'
    ], # 依赖包
    python_requires='>=3.7',# python 的版本
    include_package_data=True, #
    install_package_data=True,
)
